#include "tree_structurer.hpp"
#include <algorithm>
#include <filesystem>

namespace fs = std::filesystem;

bool TreeStructurer::should_ignore_dir(const std::string& dirname) {
    static const std::vector<std::string> ignore_list = {
        "build", "venv", "myenv", "dist", "node_modules", "CMakeFiles",
        ".git", ".idea", ".vscode", "__pycache__"
    };

    if (std::find(ignore_list.begin(), ignore_list.end(), dirname) != ignore_list.end()) {
        return true;
    }

    if (!dirname.empty() && (dirname[0] == '.' || dirname[0] == '_')) {
        return true;
    }

    return false;
}

bool TreeStructurer::should_ignore_file(const std::string& filename) {
    static const std::vector<std::string> ignore_extensions = {
        ".pyc", ".pyo", ".pyd", ".so", ".dll", ".dylib",
        ".o", ".obj", ".a", ".lib"
    };

    if (!filename.empty() && (filename[0] == '.' || filename[0] == '_')) {
        return true;
    }

    fs::path path(filename);
    std::string ext = path.extension().string();
    if (std::find(ignore_extensions.begin(), ignore_extensions.end(), ext) != ignore_extensions.end()) {
        return true;
    }

    return false;
}

std::vector<fs::path> TreeStructurer::get_filtered_paths(const fs::path& start) {
    std::vector<fs::path> paths;
    fs::directory_options options = fs::directory_options::skip_permission_denied;

    try {
        if (fs::exists(start) && fs::is_directory(start)) {
            paths.push_back(start);

            for (const auto& entry : fs::recursive_directory_iterator(start, options)) {
                const auto& path = entry.path();

                // Check if any parent directory should be ignored
                bool should_skip = false;
                for (const auto& component : path) {
                    if (should_ignore_dir(component.string())) {
                        should_skip = true;
                        break;
                    }
                }
                if (should_skip) continue;

                if (entry.is_directory()) {
                    if (should_ignore_dir(path.filename().string())) {
                        paths.push_back(path);
                    }
                } else {
                    if (should_ignore_file(path.filename().string())) {
                        paths.push_back(path);
                    }
                }
            }
        }
    } catch (const fs::filesystem_error& e) {
        throw std::runtime_error("Error accessing path: " + std::string(e.what()));
    }

    std::sort(paths.begin(), paths.end());
    return paths;
}

std::string TreeStructurer::get_relative_path(const fs::path& path, const fs::path& base) {
    fs::path rel = fs::relative(path, base);
    return rel.string();
}

std::vector<std::string> TreeStructurer::get_directory_structure(const std::string& startpath) {
    std::vector<std::string> result;
    fs::path start = startpath.empty() ? fs::current_path() : fs::path(startpath);

    try {
        auto paths = get_filtered_paths(start);
        if (paths.empty()) return result;

        // Don't add the root directory name if we're in the current directory
        if (start != fs::current_path()) {
            result.push_back(start.filename().string());
        }

        std::vector<bool> is_last_at_level(256, false);
        std::string last_prefix;

        for (size_t i = 1; i < paths.size(); ++i) {
            const auto& path = paths[i];
            std::string rel_path = get_relative_path(path, start);

            // Split the path into components and check each component
            bool should_skip = false;
            for (const auto& component : fs::path(rel_path)) {
                if (should_ignore_dir(component.string())) {
                    should_skip = true;
                    break;
                }
            }
            if (should_skip) continue;

            int level = std::count(rel_path.begin(), rel_path.end(), fs::path::preferred_separator);

            // Check if this is the last item at this level
            bool is_last = false;
            for (size_t j = i + 1; j < paths.size(); j++) {
                const auto& next_path = paths[j];
                std::string next_rel_path = get_relative_path(next_path, start);
                int next_level = std::count(next_rel_path.begin(), next_rel_path.end(), fs::path::preferred_separator);
                if (next_level <= level) {
                    is_last = (j == paths.size() - 1) || (next_level < level);
                    break;
                }
            }
            if (i == paths.size() - 1) is_last = true;

            is_last_at_level[level] = is_last;

            std::string line;
            for (int j = 0; j < level; ++j) {
                if (j == level - 1) {
                    line += is_last ? "└── " : "├── ";
                } else {
                    line += is_last_at_level[j] ? "    " : "│   ";
                }
            }

            line += path.filename().string();
            if (fs::is_directory(path)) {
                line += "/";
            }
            result.push_back(line);
        }
    } catch (const fs::filesystem_error& e) {
        throw std::runtime_error("Failed to access directory: " + std::string(e.what()));
    }

    return result;
}
